<CsoundSynthesizer>
<CsOptions>
-o out.wav -d --messagelevel=2 -j 4
</CsOptions>
; ==============================================
<CsInstruments>

sr	=	48000
ksmps	=	4
nchnls	=	2
0dbfs	=	1

; Include utility and 3D UDOs
#include "m_segk.udo"
#include "m_3d_place_source.udo"
#include "m_3d_global_reverb.udo"

; Include scene specific UDOs
#include "m_ball.udo"
#include "m_bar.udo"
#include "m_pipe.udo"
#include "m_wind.udo"
#include "m_flute.udo"
#include "m_scene_pneumatic_break.udo"

; Sine ftable for general purpose
gisine ftgenonce 0, 0, 32768, 10, 1

; Room parameter function
giroomfn ftgen 0, 0, -23, -23, "room_out.ftable"

; Coordinates of the metal ball sphere
gispherefn ftgen 0, 0, -108, -23, "sphere.ftable"
giSpherePitch[] init 36

; Table for the wires between the metal spheres
giwirefn ftgen 0, 0, -126, -23, "wires.ftable"
; Sizes of the wires
giWires[] init 42

; The bars along the walls
giLeftBarsY[] init 16
giRightBarsY[] init 16
giLeftBarsPitch[] init 16
giRightBarsPitch[] init 16

; Configuration of the pipes
gkPipeX[] init 8 ; X coordinates of the pipes
gkPipeY[] init 8 ; Y coordinates of the pipes
gkPipePitch[] init 8 ; Pitches of the pipes

; Global constants
; PI
gipi init 3.141592653589793
; Centre of the tennis ball circle on the x-y plane
giBallCentreX init 10
giBallCentreY init 12
giBallCentreZ init 11
giBallRadius init 9.5
gkRingFreq init .4 ; speed of playing a ring in Hertz
gkRingDev init 0 ; deviance of playing a ring
gkWindY init 0 ; y position of the windi
gkWindStrength init 0 ; overall strength of the wind
gaLeft init 0 ; left master out
gaRight init 0 ; right master out
gaSound init 0 ; input for the global reverb

; Turn on the Master output
alwayson "MasterOut"

; Play the table tennis ball
; p4 = idegree, p5 = ispeed, p6 = ivolume
instr BounceTennis
	idegree init p4
	ispeed init p5
	ivolume init p6
	isize init .5 + gauss(.5) ; Ball size
	iRadian = idegree / 180 * gipi ; Degree in radians
	kBounce init 0 ; bounce count of the ball
	kTrigger metro (ispeed / (giBallRadius / 4 * gipi))
	kVolume init ivolume
	iTime init (giBallRadius * gipi) / ispeed

	; Stop after four ball hits
	if kTrigger == 1 then
		kVolume = ivolume - (ivolume * .125 * kBounce)
		kBounce += 1
		if kBounce > 4 then
			turnoff
		endif
	endif

	; Coordinates of the ball
	kAngle line 0, iTime, gipi
	kx line giBallCentreX + (sin(iRadian) * giBallRadius), iTime, giBallCentreX + (sin(iRadian + gipi) * giBallRadius)
	ky line giBallCentreY + cos(iRadian) * giBallRadius, iTime, giBallCentreY + cos(iRadian + gipi) * giBallRadius
	kz = giBallCentreZ + sin(kAngle) * giBallRadius

	; Play the ball and position it
	aBall m_ball isize, kTrigger
	aBall *= kVolume
	gaSound += aBall * .15 ; for the global reverb

	aLeft, aRight m_3d_place_source aBall, kx, ky, kz, giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	gaLeft += aLeft
	gaRight += aRight
endin

; Shoot or spew a ball
;p4 idegree around the circle
instr ShootBall
	iRadian init p4 / 180 * gipi
	ix init giBallCentreX + (sin(iRadian) * giBallRadius)
	iy init giBallCentreY +(cos(iRadian) * giBallRadius)
	iz init giBallCentreZ
	isize = .55 + gauss(.05)

	aWoosh m_scene_pneumatic_break isize, .2
	aLeft, aRight m_3d_place_source aWoosh*.5, ix, iy, iz, giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	gaSound += aWoosh*.125
	gaLeft += aLeft
	gaRight += aRight
endin

; Shoot and bounce a ball
;p4 = idegree around the centre circle from where the ball comes
instr PlayTennis
	idegree init p4
	ispeed init 36 + linrand(108)
	ishootTime init 1 + ispeed / 36
	ivolume init .7 + linrand(.3)
	event_i "i", "ShootBall", 0, ishootTime, idegree
	event_i "i", "BounceTennis", (ishootTime - .1), 3, idegree, ispeed, ivolume
	turnoff
endin

; Set up pitches for the metal balls
instr SetupSpheres
	iIndex init 0
	while iIndex < 36 do
		giSpherePitch[iIndex] init 128 + linrand(128)
		iIndex += 1
	od
	turnoff
endin

; Strike one ball
; p4 = iSphere
instr StrikeSphere
	iBallNumber init p4
	iPitch init giSpherePitch[iBallNumber]
	ix init giBallCentreX + tab_i((iBallNumber * 3), gispherefn, 0) * giBallRadius
	iy init giBallCentreY + tab_i((iBallNumber * 3 + 1), gispherefn, 0) * giBallRadius
	iz init giBallCentreZ + tab_i((iBallNumber * 3 + 2), gispherefn, 0) * giBallRadius
	iForce random .2, .5

	aBall m_bar iForce, iPitch, 0, .1
	gaSound += aBall * .22

	aLeft, aRight m_3d_place_source aBall, ix, iy, iz, giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	gaLeft += aLeft
	gaRight += aRight
endin

; Instrument to play a ring
; This is controlled by gkRingFreq and gkRingDev
; p4 = iRingNumber (0-5), p5 = iDirection (1=forward, -1=backward
instr PlayRing
	iRingNumber init p4
	iDirection init p5
	; Position in the ring
	if iDirection == 1 then
		kPosition init 0
	else
		kPosition init 5
	endif
	kSpherePos init 0 ; Position index in the sphere table
	
	; Get the trigger to play a metal ball
	kTrigger gausstrig 1, gkRingFreq, gkRingDev, 1

	if kTrigger == 1 then
		kSpherePos = (iRingNumber * 6 + kPosition)
		event "i", "StrikeSphere", 0, 3, kSpherePos
		kPosition += iDirection
		kPosition wrap kPosition, 0, 6
	endif
endin

; Instrument to set ring playing frequency and deviation
instr ControlRings
	kNewFreq init i(gkRingFreq)
	kChangeFreq = gkRingFreq / 5
	kChangeTrig gausstrig 1, kChangeFreq, .5

	; Values for the random frequency change
	kmin line .5, p3, 1
	kmax line 1.25, p3, 2
	gkRingDev line .5, p3, 0

	if kChangeTrig == 1 then
		kNewFreq *= random(kmin,kmax)
		kNewFreq limit kNewFreq, .5, 5
	endif
	gkRingFreq port kNewFreq, 3
endin

; Move the wind position
instr ControlWind
	; Control the strength of the wind
	gkWindStrength oscil .125, .2, gisine
	gkWindStrength += .75

	; Control the wind position
	ifreq init .1 ; frequency of events
	kfreq line ifreq, p3, ifreq/2
	krear line 0, p3, (tab_i(3,giroomfn,0) - 25) ; maximum rear position of the wind
	kstage init 0 ; 0 go to rear, 1 go to front
	ktrigger metro kfreq
	kpos init 2.2 ; position of the wind

	if ktrigger == 1 then
		if kstage == 0 then
			kpos = krear + linrand(2 + (giBallRadius * 2))
			kstage = 1
		else
			kpos = linrand(giBallCentreY) + (giBallRadius * .75)
			kstage = 0
		endif
		kpos += 2.2
	endif
	gkWindY port kpos, .15/ifreq, i(gkWindY)
endin

; Fade out the wind, move it towards the back
instr WindFade
	ipos init i(gkWindY)
	gkWindY line ipos, p3, 50
endin

; Instrument to play all the rings
instr PlayRings
	ktime timeinsts
	kTrigger metro gkRingFreq/6 ; trigger for new events
	kRingCount init 0

	if kRingCount <= 4 then ; Only play new rings until all are sounding
		if kTrigger == 1 then
			kdelay = 1.75 / gkRingFreq
			event "i", "PlayRing", 0, (p3-ktime), kRingCount, 1
			event "i", "PlayRing", kdelay, (p3 - ktime - kdelay), (kRingCount + 1), -1
			kRingCount += 2
		endif
	endif
endin

; Play the whole metal ball sphere randomly
; p4 = iFreq - frequency of the srikes
instr PlaySphere
	ifreq init p4
	kdeviat oscil .5, gkRingFreq, gisine
	kdeviat += .5
	ksphere init 0
	kTrigger gausstrig 1, ifreq, kdeviat, 1

	if kTrigger == 1 then
		ksphere linrand 36
		ksphere int ksphere
		event "i", "StrikeSphere", 0, 3, ksphere
	endif
endin

; Setup the wires connecting the metal balls
instr SetupWires
	iIndex init 0
	while iIndex <42 do
		giWires[iIndex] = .5 + gauss(.5)
		iIndex += 1
	od
	turnoff
endin

; Play one wire based on the wind position
;p4 = iWireNumber
instr PlayWire
	iWireNumber init p4
	ix = tab_i((iWireNumber * 3), giwirefn, 0) * giBallRadius + giBallCentreX
	iy = tab_i((iWireNumber * 3 + 1), giwirefn, 0) * giBallRadius + giBallCentreY
	iz = tab_i((iWireNumber * 3 + 2), giwirefn, 0) * giBallRadius + giBallCentreZ
	kstrength init 0
	kprox init 3
	isize init giWires[iWireNumber]
	aLeft init 0
	aRight init 0

	; Strnegth based on proximity of wind
	kprox = abs(iy - gkWindY)
	if (kprox <= 4) then
		kstrength = (4 - kprox) / 4
		aWire m_wind (kstrength*gkWindStrength), isize
		gaSound += aWire * .3
		aLeft, aRight m_3d_place_source aWire*3.75, ix, iy, iz, giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	else
		aLeft = 0
		aRight = 0
	endif

	gaLeft += aLeft
	gaRight += aRight
endin

; Make the wires sing, when the wind blows
instr PlayWires
	; Start all wires
	iIndex init 0
	while iIndex <42 do
		event_i "i", "PlayWire", 0, p3, iIndex
		iIndex += 1
	od
	turnoff
endin

; Setup the metal bars
instr SetupBars
	iIndex init 0

	while iIndex <16 do
		giLeftBarsY[iIndex] = (iIndex * 3) + .5
		giRightBarsY[iIndex] = (iIndex * 3) + 2
		giLeftBarsPitch[iIndex] = linrand(32) + 16
		giRightBarsPitch[iIndex] = linrand(31) + 17
		iIndex += 1
	od
	turnoff
endin

; Strike one metal bar
; p4 = iSide (0=left, 1=right), p5 = iBarNumber
instr StrikeBar
	iSide init p4
	iBarNumber init p5
	iforce = linrand(.3) + .7
	iscan = linrand(.5) + .25
	idamp = linrand(.1) + .1
	iz init 2

	if iSide == 0 then ; left
		aBar m_bar iforce, giLeftBarsPitch[iBarNumber], idamp, iscan
		ix init 2
		iy init giLeftBarsY[iBarNumber]
	else ; right side
		aBar m_bar iforce, giRightBarsPitch[iBarNumber], idamp, iscan
		ix = tab_i(2, giroomfn, 0) - 2
		iy init giRightBarsY[iBarNumber]
	endif
	gaSound += aBar * .3

	; Place the sound
	aLeft, aRight m_3d_place_source aBar, ix, iy, iz, giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	gaLeft += aLeft
	gaRight += aRight
endin

; Play the bars
;p4 = iFreq, p5 = iDirection (0=forwards, 1=backwards)
instr PlayBars
	ifreq init p4
	iDirection init p5
	kTrigger metro ifreq
	iDelay init .5 / ifreq
	kIndex init 0 ; Index into the bars

	if kTrigger == 1 then
		if iDirection == 0 then
			kBar = kIndex
		else
			kBar = (15 - kIndex)
		endif
		event "i", "StrikeBar", 0, 1.5, 0, kBar
		event "i", "StrikeBar", iDelay, 1.5, 1, kBar
		kIndex += 1
	endif

	if kIndex == 16 then
		turnoff
	endif
endin

; Play a note on the flute
;p4 = ipitch, p5 = iSide (0=left 1=right)
instr PlayFlute
	ipitch init p4
	iSide init p5
	aLeft init 0
	aRight init 0

	aFlute m_flute ipitch
	gaSound += aFlute*.01
	aFlute *= .06
	if iSide == 0 then
		aLeft = aFlute
	else
		aRight = aFlute
	endif
	gaLeft += aLeft
	gaRight += aRight
endin

; Play the cuckoo
instr Cuckoo
	iSide init round(linrand(1.1))
	event_i "i", "PlayFlute", 0, .2, 900, iSide
	event_i "i", "PlayFlute", .25, .2, 750, iSide
	turnoff
endin

; Setup the pipes
instr SetupPipes
	iIndex init 0
	while iIndex <8 do
		if (iIndex % 2) == 0 then
			gkPipeX[iIndex] init giBallCentreX - (2 * (iIndex + 1))
		else
			gkPipeX[iIndex] init giBallCentreX + (2 * (iIndex + 1))
		endif
		gkPipeY[iIndex] = 1
		gkPipePitch[iIndex] init linrand(.06)
		iIndex += 1
	od
	turnoff
endin

; Play a big pipe
;p4 = iPipeNumber
instr PlayPipe
	iPipeNumber init p4
	iGain init 2 + gauss(.4)

	; get the sound
	aPipe m_pipe gkPipePitch[iPipeNumber]
	aPipe *= iGain
	gaSound += aPipe*.2

	aLeft, aRight m_3d_place_source aPipe, gkPipeX[iPipeNumber], gkPipeY[iPipeNumber], \
		(giBallCentreZ + 1.5 ), giBallCentreX, giBallCentreY, giBallCentreZ, \
		0, giroomfn
	gaLeft += aLeft
	gaRight += aRight
endin

; Strike a pipe to bring the others to heel
instr StrikePipe
	kamp expon .4, p3, .00001
	aNoise fractalnoise kamp, 1
	aPipe m_bar .1, 8, .1, 0
	aLNoise butterlp (aNoise * .1), 600
	aLPipe mvclpf3 aPipe, 2000, .9
	aSound = (aLNoise*.4 + aLPipe) * 5
	gaSound += aSound * .1

	aLeft, aRight m_3d_place_source aSound, gkPipeX[0], gkPipeY[0], (giBallCentreZ -1.5), \
		giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	gaLeft += aLeft
	gaRight += aRight
endin

; Play all pipes in rhythm
;p4 = iFrequency - tempo of the pipes
instr PlayPipes
	ifreq init p4
	kDriftFreq init .1

	; Create deviation/rhythm drifting over time
	iramp ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	kHeel init 1 ; trigger to bring pipes back to rhythm
	kdev oscil .25, kDriftFreq, iramp
	kCuckoo init 7 ; Pipe that will trigger the cuckoo

	kTrigger gausstrig 1, ifreq, kdev, 1

	kIndex init 0
	if kTrigger == 1 then
		event "i", "PlayPipe", 0, .4, kIndex
		if kIndex == 7 then
			kIndex = 0
		else
			kIndex += 1
		endif
	endif

	; Play the cuckoo
	if (abs(giBallCentreY - gkPipeY[kCuckoo]) <.1) then ;&& (gkPipeX[kCuckoo] >giBallCentreX) then
		schedkwhen 1, 25, 1, "Cuckoo", 0, .1
		if kCuckoo == 0 then
			kCuckoo = 7
		else
			kCuckoo -= 1
		endif
	endif

	; Bring the pipes back to heel
	if kdev >= .199 then
		if kHeel == 1 then
			event "i", "StrikePipe", 0, .8
			event "i", "StrikePipe", .5/ifreq, .8
			kHeel = 0
			kDriftFreq = .1 + gauss(.05)
		endif
	elseif kdev <= .1 then
		kHeel = 1
	endif
endin

; Move one pipe
;p4 = iPipeNumber
instr MovePipe
	iPipeNumber init p4
	if (iPipeNumber % 2) == 0 then ; even pies left
		kx init giBallCentreX - (2 * (iPipeNumber +1))
	else ; odd ones go right
		kx init giBallCentreX + (2 * (iPipeNumber +1))
	endif
	ky init 1
	kfin init 0
	kfin2 init 0
	ktrig init 1

	; Function for the move
	imovexfn ftgenonce 0, 0, 4096, 6, 0, 1536, 0, 512, 1, 1536, 1, 512, 0
	imoveyfn ftgenonce 0, 0, 4096, 7, 0, 2048, 1, 2048, 0

	if kfin == 0 then ; bring them into line
		if (iPipeNumber % 2) == 0 then ; even pipes left
			kx, kfin m_segk gkPipeX[iPipeNumber], 10, (giBallCentreX - 7), 1, ktrig
		else ; odd pipes right
			kx, kfin m_segk gkPipeX[iPipeNumber], 10, (giBallCentreX + 7), 1, ktrig
		endif
		ky, kfin2 m_segk gkPipeY[iPipeNumber], 10, 6, 1, ktrig
	else ; move in normal procession
		kx oscil 14, 1/30, imovexfn
		if (iPipeNumber % 2) == 0 then ; even pipes
			kx = (giBallCentreX - 7) + kx
		else ; odd pipes
			kx = (giBallCentreX + 7) - kx
		endif
		ky oscil 35, 1/30, imoveyfn
		ky += 6
	endif

	gkPipeX[iPipeNumber] = kx
	gkPipeY[iPipeNumber] = ky

	ktrig = 0
endin

; Move all pipes
;p4 = iOffset - offset between the pipes
instr MovePipes
	ktime timeinsts
	kIndex init 0
	ifreq init p4
	kTrigger metro ifreq

	if kTrigger == 1 then
		event "i", "MovePipe", 0, (p3 - ktime), kIndex
		kIndex += 1
	endif

	if kIndex == 8 then
		turnoff
	endif
endin

; Perform a spiralling frozen reverb bell-sound
;p4 = iRadius, p5 = iSize
instr PlaySpiral
	iRadius init p4
	iSize init p5
	ky expon (giBallCentreY + iRadius), p3, (tab_i(3, giroomfn, 0) - 1)

	; Simple ramp function
	iRamp ftgenonce 0, 0, 4096, 7, 0, 4096, 1
	kSpinFreq line .5, p3, 10 ; The spiral gets faster
	kAngle oscil (2*gipi), kSpinFreq, iRamp ; The actual angle of the bell
	kx = giBallCentreX + sin(kAngle) * iRadius
	kz = giBallCentreZ + cos(kAngle) * iRadius
	kdistance = sqrt((ky - giBallCentreY)*(ky - giBallCentreY) + iRadius*iRadius)
	; Gain of the sound (should rise with distance)
	kGain linseg .75, p3-.5, .75, .001, 3, .499, .7

	; Produce the sound
	iforce init linrand(1)
	ipitch init iSize + linrand(iSize)
	idamp init 0
	kscan init .15
	aBar m_bar iforce, ipitch, idamp, kscan

	; Reverb
	kFeedback linseg .8, .5, 1, p3-2, 1, .1, .9, 1.4, .9
	aRevL, aRevR reverbsc aBar, aBar, kFeedback, 8000

	; Now get the doppler effect
	aDoppler doppler (aBar + aRevL), kdistance, 0
	aSource = (aBar*.3 + aDoppler)*kGain
	gaSound += aSource*.2

	; Place in space
	aLeft, aRight m_3d_place_source aSource, kx, ky, kz, \
		giBallCentreX, giBallCentreY, giBallCentreZ, 0, giroomfn
	gaLeft += aLeft
	gaRight += aRight
endin

; Intro section tennis balls and one or two spirals
instr Intro
	iTennisFreq init 5/p3
	iSpiralFreq init 2/p3
	kdegree init 0
	kradius init 0
	ksize init 0
	kdur init 0
	kTennisTrig gausstrig 1, iTennisFreq, .7, 0, 1
	kSpiralTrig gausstrig 1, iSpiralFreq, .7, 0, 1

	iballNumber init 18 + int(gauss(18))
	event_i "i", "StrikeSphere", 0, 3, iballNumber
	if kTennisTrig == 1 then
		kdegree linrand 360
		event "i", "PlayTennis", 0, 5, kdegree
	endif

	if kSpiralTrig == 1 then
		kradius = 5 + linrand(4.9)
		ksize = 12 + linrand(8)
		kdur = 12 + linrand(15)
		event "i", "PlaySpiral", 0, kdur, kradius, ksize
	endif
endin

; Play tennis balls as fireworks
;p4 = ifreq - frequency at which a ball is fired
instr Fireworks
	ifreq init p4
	kdegree init 0
	ktrigger gausstrig 1, ifreq, .5, 0, 1
	if ktrigger == 1 then
		kdegree = linrand(360)
		event "i", "PlayTennis", 0, 5, kdegree
	endif
endin

; Let the metal bars applaud
instr Applaud
	ktrigger gausstrig 1, 1/20, .5, 0, 1
	kPlayFreq init 4
	kPlayDirection init 0

	if ktrigger == 1 then
		kPlayFreq = 2 + linrand(2)
		event "i", "PlayBars", 0, 17/kPlayFreq, kPlayFreq, kPlayDirection
		if kPlayDirection == 0 then
			kPlayDirection = 1
		else
			kPlayDirection = 0
		endif
	endif
endin

; Global reverb (late reflections) instrument
instr Reverb
	aLeft, aRight m_3d_global_reverb gaSound, giroomfn
	gaLeft += aLeft
	gaRight += aRight
	gaSound = 0
endin

; Master output instrument
instr MasterOut
	outs gaLeft*6, gaRight*6
	gaLeft = 0
	gaRight = 0
endin

</CsInstruments>
; ==============================================
<CsScore>
i"SetupSpheres" 0 .1
i"SetupWires" 0 .1
i"SetupBars" 0 .1
i"SetupPipes" 0 .1
i"Intro" .1 30
i"PlayRing" 31 15 1 1
i"ControlWind" 31 209
i"PlayWires" 31 225
i"ControlRings" 50 30
i"PlayRings" 50 45
i"PlaySphere" 96 24 5
i"PlaySpiral" 119 15 8 32
i"PlayPipes" 119 121 3 ; Play pipes three times per second
i"MovePipes" 130 110 1
i"Fireworks" 130 109 .05
i"PlaySphere" 130 120 .125
i"Applaud" 130 109 
i"PlaySpiral" 128 20 9 24
i"PlaySpiral" 185 25 6 21
i"PlaySpiral" 238 20 7 12
i"WindFade" 240 20
i"Reverb" 0 265
e
</CsScore>
</CsoundSynthesizer>
