/*
m_3d_global_reverb - put global hrtf reverb on an input signal

DESCRIPTION
m_3d_reverb reverberates a signal in a predefined room. This only adds the late
reverb using hrtfreverb.
This UDO extends the duration (p3) of an instrument by the longest reverb time
(RT60), so there is no need to add extra time after the end of the last event.

SYNTAX
aleft, aright m_3d_global_reverb ain, iroomfn

INITIALIZATION
iroomfn - ftable describing a room
	Format of room ftable:
	fade: 1-24 (lower values recommended for complex signals/trajectories)
	order: order of processing 0-4 (2 seems OK)
	x: width of the room in metres (left to right)
	y: length of the room in metres (front to back)
	z: height of the room in metres (bottom to top)
	wallhi: high frequency absorption coefficient of the walls (0-1)
	walllo: low frequency absorption coefficient of the walls (0-1)
	wallgain1: EQ of the walls at 250Hz, Q = 4 octaves (0-1)
	wallgain2: EQ of the walls at 1000Hz, Q = 4 octaves (0-1)
	wallgain3: EQ of the walls at 4000Hz, Q = 4 octaves (0-1)
	floorhi: see wallhi above
	floorlo: see wallllo above
	floorgain1: see wallgain1 above
	floorgain2: see wallgain2 above
	floorgain3: see wallgain3 above
	ceilinghi: ...
	ceilinglo: ...
	ceilinggain1: ...
	ceilinggain2: ...
	ceilinggain3: ...
	rtlow: reverb time for low frequencies (given by hrtfearly or autogenerated
		by m_3d_test_room.csd)
	rthigh: reverb time for high frequencies (given by hrtfearly or autogenerated
		by m_3d_test_room.csd)
	mpf: mean free path (given by hrtfearly or autogenerated by
		m_3d_test_room.csd)

PERFORMANCE
ain - input signal, usually a global mono signal, which has also been positioned
	 in the same room using hrtfearly

CREDITS
Author: Jeanette C., thanks to Brian Carty, author of the HRTF opcodes.
*/

opcode m_3d_global_reverb, aa, ai
	arevin, iroomfn xin

	; Read reverbaration parameters
	iorder table 1, iroomfn, 0
	irtl table 20, iroomfn, 0
	irth table 21, iroomfn, 0
	imfp table 22, iroomfn, 0

	; Extend this instruments duration by the longest RT60
	if irtl >= irth then
		xtratim irtl
	else
		xtratim irth
	endif

	Sleft sprintf "hrtf-%d-left.dat", sr
	Sright sprintf "hrtf-%d-right.dat", sr

	; Reverberate the signal and delay it
	al, ar, idel hrtfreverb arevin, irtl, irth, Sleft, Sright, sr, imfp, iorder
	aleft delay al*.1, idel
	aright delay ar*.1, idel

	xout aleft, aright
endop
