/*
m_3d_place_source - place a source in a 3d room

DESCRIPTION
Place a monaural audio source in a predefined room.

SYNTAX
aleft, aright m_3d_place_source asrc, ksrcx, ksrcy, ksrcz, klistenerx, \
		klistenery, klistenerz, kheadrot, iroomfn

INITIALIZATION
iroomfn - ftable defining the room
	Format of room ftable:
	fade: 1-24 (lower values recommended for complex signals/trajectories)
	order: order of processing 0-4 (2 seems OK)
	x: width of the room in metres (left to right)
	y: length of the room in metres (front to back)
	z: height of the room in metres (bottom to top)
	wallhi: high frequency absorption coefficient of the walls (0-1)
	walllo: low frequency absorption coefficient of the walls (0-1)
	wallgain1: EQ of the walls at 250Hz, Q = 4 octaves (0-1)
	wallgain2: EQ of the walls at 1000Hz, Q = 4 octaves (0-1)
	wallgain3: EQ of the walls at 4000Hz, Q = 4 octaves (0-1)
	floorhi: see wallhi above
	floorlo: see wallllo above
	floorgain1: see wallgain1 above
	floorgain2: see wallgain2 above
	floorgain3: see wallgain3 above
	ceilinghi: ...
	ceilinglo: ...
	ceilinggain1: ...
	ceilinggain2: ...
	ceilinggain3: ...
	rtlow: reverb time for low frequencies (given by hrtfearly or autogenerated
		by m_3d_test_room.csd)
	rthigh: reverb time for high frequencies (given by hrtfearly or autogenerated
		by m_3d_test_room.csd)
	mpf: mean free path (given by hrtfearly or autogenerated by
		m_3d_test_room.csd)

PERFORMANCE
aleft, aright - left and right output
asrc - audio source input
ksrcx, ksrcy, ksrcz - x, y and z coordinates of the audio source (measured
	from the front left bottom corner)
klistenerx, klistenery, klistenerz - x, y and z coordinates of the listener
	(measured from front left bottom corner of the room)
kheadrot - head rotation of the listener in degrees

CREDITS
Author: Jeanette C., thanks to Brian Carty author of the Csound HRTF opcodes.
*/

opcode m_3d_place_source, aa, akkkkkkki
	asrc, ksx, ksy, ksz, klx, kly, klz, kheadrot, iroomfn xin

	; Read basic room parameters
	irfade table 0, iroomfn, 0
	irorder table 1, iroomfn, 0
	irx table 2, iroomfn, 0
	iry table 3, iroomfn, 0
	irz table 4, iroomfn, 0
	irwallhi table 5, iroomfn, 0
	irwalllo table 6, iroomfn, 0
	irwallg1 table 7, iroomfn, 0
	irwallg2 table 8, iroomfn, 0
	irwallg3 table 9, iroomfn, 0
	irfloorhi table 10, iroomfn, 0
	irfloorlo table 11, iroomfn, 0
	irfloorg1 table 12, iroomfn, 0
	irfloorg2 table 13, iroomfn, 0
	irfloorg3 table 14, iroomfn, 0
	irceilhi table 15, iroomfn, 0
	irceillo table 16, iroomfn, 0
	irceilg1 table 17, iroomfn, 0
	irceilg2 table 18, iroomfn, 0
	irceilg3 table 19, iroomfn, 0
	Sleft sprintf "hrtf-%d-left.dat", sr
	Sright sprintf "hrtf-%d-right.dat", sr

	; Position the source in the room
	aleft, aright, irtl, irth, imfp hrtfearly asrc, ksx, ksy, ksz, klx, kly, klz, Sleft, Sright, 0, irfade, sr, irorder, 1, kheadrot, irx, iry, irz, irwallhi, irwalllo, irwallg1, irwallg2, irwallg3, irfloorhi, irfloorlo, irfloorg1, irfloorg2, irfloorg3, irceilhi, irceillo, irceilg1, irceilg2, irceilg3

	xout aleft, aright
endop
