/*
m_scene_pneumatic_break - semi-physical model of a penumatic break

DESCRIPTION
This UDO emulates the sound of a penumatic break, as can be found in subway
trains and similar vehicles. Although physical models have been used, it is
more a sound-alike than a physical model.

SYNTAX
aout m_scene_pneumatic_break isize, iattack

INITIALIZATION
isize - Size of the system, between 0 and 1
iattack - Time for the volume to reach maximum, sounds good at 0.02

PERFORMANCE

CREDITS
Author: Jeanette C.
*/

opcode m_scene_pneumatic_break, a,ii
	isize, iattack xin

	xtratim .1 ; add time for the linsegr
	isine ftgenonce 0, 0, 32768, 10, 1 ; sine ftable for vibrato
	iinvsize init (1 - isize) ; inverted isize for convenience
	kamp linsegr 0, iattack, 1, p3-(.3 + iattack), 1, .3, 0, .1, 0
		; overall amp of the signal
	kfamp linseg 1, p3-.1, 1, .1, .001 ; amp to be given to the flute model
	kngain linseg .8, p3*.8, .5, p3/5, .3 ; gain of the noise component
	iminfreq linlin iinvsize, 110, 220 ; minimum freq/size of the pipe?
	aflute wgflute kfamp, 220, .04, .2, .05, kngain, .1, 0, isine, \
		iminfreq, .3, .75 ; the basic sound
	afdc dcblock2 aflute ; use DC offset filter, since that model can generate
	afdc = afdc * (1.9 + (iinvsize * 2)) * kamp ; scale sound
		; a high DC value
	
	; Sort of an EQ to make the result sound better
	aoutlp butterlp afdc, 5500
	alp1 butterlp afdc, 3520
	aout1 butterhp alp1, 1760
	alp2 butterlp afdc, 660
	aout2 butterhp alp2, 330
	aout3 butterhp afdc, 10560
	afilt = aout1*.35 + aout2*1.5 + aout3 ; sum the EQ bands
	adry = (afilt * iinvsize) + (aoutlp * isize) ; balance EQ'ed and unEQ'ed
	aout wguide1 adry, 220, 5500, .5*isize ; pass through an extra waveguide
		; model to give more impression of space/size
	xout aout
endop
